//valac --pkg libgdata --pkg granite --pkg webkitgtk-3.0 gdata.vala

using GData;
using WebKit;

namespace GWoffice
{
	
	public enum DocType
	{
		DOCUMENT,
		SPREADSHEET,
		PRESENTATION
	}
	
	public class App : Granite.Application
	{
		
		construct
		{
			program_name = "GWoffice";
			exec_name = "gwoffice";
			
			build_data_dir = Constants.DATADIR;
			build_pkg_data_dir = Constants.PKGDATADIR;
			build_release_name = Constants.RELEASE_NAME;
			build_version = Constants.VERSION;
			build_version_info = Constants.VERSION_INFO;
			
			app_years = "2012";
			app_icon = exec_name;
			app_launcher = exec_name+".desktop";
			application_id = "net.launchpad."+exec_name;
			
			main_url = "https://code.launchpad.net/"+exec_name;
			bug_url = "https://bugs.launchpad.net/"+exec_name;
			help_url = "https://code.launchpad.net/"+exec_name;
			translate_url = "https://translations.launchpad.net/"+exec_name;
			
			about_authors = {"Tom Beckmann <tombeckmann@online.de>"};
			about_documenters = {"Tom Beckmann"};
			about_artists = {"Tom Beckmann"};
			about_comments = "Access your google drive from the desktop";
			about_translators = "";
			about_license_type = Gtk.License.GPL_3_0;
		}
		
		public bool ubuntu_style;
		
		internal Gtk.Window window;
		
		internal DocumentsService docs;
		internal Widgets.FileView file_view;
		internal Widgets.DynamicNotebook notebook;
		internal Granite.Widgets.Welcome welcome;
		internal Services.HUD hud;
		internal Services.Sync sync;
		
		//progress bar
		public Gtk.ToolItem progress_item;
		public Gtk.Label progress_label;
		public Gtk.ProgressBar progress_bar;
		
		//context actions
		Gtk.ToolButton trash;
		Gtk.ToolButton download;
		Gtk.ToolButton access;
		
		internal File login_data_file;
		internal File cookie_file;
		internal File sync_dir;
		internal File etags_file;
		internal File data_folder;
		
		public signal void authorization_done (Authorizer auth);
		bool already_authenticated = false;
		bool authenticated = false;
		bool opening = false;
		
		public Unity.LauncherEntry launcher;
		
		//total count of all docs that belong to me, might not be perfectly exact, but at least a good estimate
		public int total_count = 0;
		
		public App ()
		{
			flags = ApplicationFlags.HANDLES_OPEN;
			
			launcher = Unity.LauncherEntry.get_for_desktop_id (app_launcher);
			launcher.quicklist = new Dbusmenu.Menuitem ();
			
			data_folder = File.new_for_path (Environment.get_user_data_dir ()+"/"+exec_name);
			if (!data_folder.query_exists ()) {
				try {
					data_folder.make_directory ();
				} catch (Error e) { warning (e.message); }
			}
			
			login_data_file = File.new_for_path (data_folder.get_path ()+"/auth");
			cookie_file = File.new_for_path (data_folder.get_path ()+"/cookies.txt");
			etags_file = File.new_for_path (data_folder.get_path ()+"/etags");
			sync_dir = File.new_for_path (Environment.get_home_dir ()+"/"+exec_name);
			
			//if we have ambiance or radiance, switch to them
			ubuntu_style = Services.Settings.get_default ().ubuntu_style == -1 || 
				(Services.Settings.get_default ().ubuntu_style == 1 && 
				(Gtk.Settings.get_default ().gtk_theme_name == "Ambiance" || Gtk.Settings.get_default ().gtk_theme_name == "Radiance"));
			
			/*create some ui stuff already*/
			window = new Gtk.Window ();
			file_view = new Widgets.FileView (this);
			progress_bar = new Gtk.ProgressBar ();
			progress_item = new Gtk.ToolItem ();
			hud = new Services.HUD (this);
			
			window.sensitive = false;
			
			if (docs == null) {
				//prepare for authorization done
				authorization_done.connect ((auth) => {
					authenticated = true;
					docs = new DocumentsService (auth);
					
					if (!opening) { //called at open callback, just upload stuff
						window.sensitive = true;
						
						progress_label.label = _("Collecting documents...");
						progress_item.show_all ();
						
						//get all the docs for normal use
						docs.query_documents_async.begin (null, null, (e, key, count) => {
							
							bool mine = false;
							e.get_categories ().foreach ((c) => {
								if (c.scheme.index_of ("http://schemas.google.com/docs/2007/folders/") != -1) {
									mine = true;
									total_count ++;
									return;
								}
							});
							
							file_view.add_item (e as DocumentsEntry, mine);
							progress_bar.fraction = (double)key/count;
							
						}, (obj, res) => {
							
							//all docs grabbed
							progress_item.hide ();
							
							sync = new Services.Sync (docs, this);
							if (Services.Settings.get_default ().download_docs &&
								Services.Settings.get_default ().resync_on_startup) {
								sync.resync ();
							}
						});
					}
				});
				
				//authentication with goa
				Authorizer? auth = null;
				try {
					var goa_accounts = new Goa.Client.sync (null).get_accounts ();
					if (false && goa_accounts != null) {
						foreach (var account in goa_accounts) {
							if (account.get_account ().provider_name == "Google")
								auth = new Services.GoaAuthorizer (goa_accounts.nth_data (0));
						}
					}
				} catch (Error e) { warning (e.message); }
				//no goa? client login perhaps?
				if (auth == null) {
					message ("GOA not available, using client login");
					auth = client_login ();
				}
				//still not? well, bye then!
				if (auth == null && !already_authenticated) {
					warning ("Could not authenticate, silly user.");
					return;
				}
				
				if (!authenticated)
					authorization_done (auth);
			}
		}
		
		public void clear_sync_folder ()
		{
			try {
				delete_rec (sync_dir);
				
				etags_file.delete ();
			} catch (Error e) { warning (e.message); }
		}
		void delete_rec (File file)
		{
			try {
				var enumerator = file.enumerate_children (FileAttribute.STANDARD_NAME, 0);
				var file_info = enumerator.next_file ();
				while (file_info != null) {
					var new_file = File.new_for_path (file.get_path ()+"/"+file_info.get_name ());
					print ("F: %s\n", new_file.get_path ());
					delete_rec (new_file);
					new_file.delete ();
					file_info = enumerator.next_file ();
				}
			} catch (Error e) { warning (e.message); }
		}
		
		public void start_ui ()
		{
			
			/*ui*/
			
			window.title = "GWoffice";
			window.window_position = Gtk.WindowPosition.CENTER;
			window.set_application (this);
			
			var box = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
			
			//welcome
			welcome = new Granite.Widgets.Welcome ("GWoffice", _("Your Google Web Office"));
			welcome.append ("x-office-document", _("Create Document"), _("Create a new Text Document"));
			welcome.append ("x-office-spreadsheet", _("Create Spreadsheet"), _("Create a new Sheet"));
			welcome.append ("x-office-presentation", _("Create Presentation"), _("Create a new Presentation"));
			
			//toolbar
			var toolbar = new Gtk.Toolbar ();
			if (ubuntu_style)
				toolbar.toolbar_style = Gtk.ToolbarStyle.BOTH;
			
			var files = new Gtk.ToolButton (new Gtk.Image.from_icon_name ((ubuntu_style)?"document-open-symbolic":"document-open", 
				Gtk.IconSize.LARGE_TOOLBAR), _("Files"));
			files.set_tooltip_text (_("Search in your files"));
			var doc = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("x-office-document", 
				Gtk.IconSize.LARGE_TOOLBAR), _("New Document"));
			doc.set_tooltip_text (_("Create a new document"));
			var sheet = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("x-office-spreadsheet", 
				Gtk.IconSize.LARGE_TOOLBAR), _("New Spreadsheet"));
			sheet.set_tooltip_text (_("Create a new spreadsheet"));
			var presentation = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("x-office-presentation", 
				Gtk.IconSize.LARGE_TOOLBAR), _("New Presentation"));
			presentation.set_tooltip_text (_("Create a new presentation"));
			var resync = new Gtk.ToolButton (new Gtk.Image.from_icon_name ((ubuntu_style)?"go-jump-symbolic":"go-down", 
				Gtk.IconSize.LARGE_TOOLBAR), _("Resync"));
			resync.set_tooltip_text (_("Resync your local files"));
			
			//appmenu
			var menu = new Gtk.Menu ();
			var delete_login_data = new Gtk.MenuItem.with_label (_("Delete login data"));
			var show_prefs = new Gtk.MenuItem.with_label (_("Preferences"));
			
			menu.append (delete_login_data);
			menu.append (show_prefs);
			
			progress_label = new Gtk.Label ("");
			progress_bar.valign = Gtk.Align.CENTER;
			progress_bar.width_request = 220;
			var prog_box = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
			prog_box.pack_start (progress_label);
			prog_box.pack_start (progress_bar, false);
			progress_item.add (prog_box);
			
			trash = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("edit-delete"+((ubuntu_style)?"-symbolic":""), Gtk.IconSize.LARGE_TOOLBAR), _("Move to trash"));
			trash.set_tooltip_text (_("Move the current document to the trash"));
			download = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("document-save-as"+((ubuntu_style)?"-symbolic":""), Gtk.IconSize.LARGE_TOOLBAR), _("Download"));
			download.set_tooltip_text (_("Download the current document"));
			access = new Gtk.ToolButton (new Gtk.Image.from_icon_name ("changes-prevent"+((ubuntu_style)?"-symbolic":""), Gtk.IconSize.LARGE_TOOLBAR), _("Access Rules"));
			access.set_tooltip_text (_("Show access rules for the current document"));
			
			var exp = new Gtk.ToolItem ();
			var exp2 = new Gtk.ToolItem ();
			exp.set_expand (true);
			exp2.set_expand (true);
			
			var app_menu = create_appmenu (menu);
			if (ubuntu_style)
				(app_menu.get_icon_widget () as Gtk.Image).icon_name = "emblem-system-symbolic";
			
			toolbar.get_style_context ().add_class ("primary-toolbar");
			toolbar.insert (files, -1);
			if (!ubuntu_style) {
				toolbar.insert (new Gtk.SeparatorToolItem (), -1);
				toolbar.insert (doc, -1);
				toolbar.insert (sheet, -1);
				toolbar.insert (presentation, -1);
			}
			toolbar.insert (new Gtk.SeparatorToolItem (), -1);
			toolbar.insert (download, -1);
			toolbar.insert (trash, -1);
			toolbar.insert (access, -1);
			toolbar.insert (exp, -1);
			toolbar.insert (progress_item, -1);
			toolbar.insert (exp2, -1);
			toolbar.insert (resync, (ubuntu_style)?1:-1);
			if (!ubuntu_style) {
				toolbar.insert (new Gtk.SeparatorToolItem (), -1);
				toolbar.insert (app_menu, -1);
			}
			
			//notebook
			notebook = new Widgets.DynamicNotebook ();
			notebook.tab_added.connect ((tab) => {
				//show popover for choosing doc type to be added
				var pop = new Widgets.PopOver ();
				var nbox = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
				
				var ndoc = new Gtk.Button.with_label (_("New Document"));
				var nsheet = new Gtk.Button.with_label (_("New Spreadsheet"));
				var npresentation = new Gtk.Button.with_label (_("New Presentation"));
				
				nbox.pack_start (ndoc);
				nbox.pack_start (nsheet);
				nbox.pack_start (npresentation);
				
				ndoc.clicked.connect (() => {create_document (DocType.DOCUMENT);pop.destroy ();});
				nsheet.clicked.connect (() => {create_document (DocType.SPREADSHEET);pop.destroy ();});
				npresentation.clicked.connect (() => {create_document (DocType.PRESENTATION);pop.destroy ();});
				
				(pop.get_content_area () as Gtk.Box).add (nbox);
				pop.move_to_widget (notebook.notebook.get_action_widget (Gtk.PackType.START));
				pop.show_all ();
				
				notebook.remove_tab (tab);//remove created tab, we will create one ourselves
			});
			
			box.pack_start (hud, false);
			box.pack_start (toolbar, false);
			box.pack_start (welcome);
			box.pack_start (notebook);
			
			//load last used settings to our window
			window.set_default_size (Services.Settings.get_default ().window_width, 
				Services.Settings.get_default ().window_height);
			
			window.add (box);
			window.set_application (this);
			window.show_all ();
			
			trash.hide ();
			download.hide ();
			access.hide ();
			notebook.hide ();
			hud.hide ();//don't actually show the menu, just use it for the HUD
			progress_item.hide ();
			
			/*events*/
			//save window state
			window.destroy.connect (() => {
				Services.Settings.get_default ().window_width = window.get_allocated_width ();
				Services.Settings.get_default ().window_height = window.get_allocated_height ();
			});
			//resync button
			resync.sensitive = Services.Settings.get_default ().download_docs;
			Services.Settings.get_default ().notify["download-docs"].connect (() => 
				resync.sensitive = Services.Settings.get_default ().download_docs
			);
			resync.clicked.connect (() => sync.resync () );
			
			//creation shortcuts
			doc.clicked.connect (() => create_document (DocType.DOCUMENT) );
			sheet.clicked.connect (() => create_document (DocType.SPREADSHEET) );
			presentation.clicked.connect (() => create_document (DocType.PRESENTATION) );
			
			//show the access options popover
			access.clicked.connect (() => {
				var document = ((notebook.current.page as Gtk.ScrolledWindow).get_child () as Widgets.WebView).doc;
				var pop = new Widgets.AccessPopOver (document, docs);
				
				pop.move_to_widget (access);
				pop.show_all ();
			});
			
			bool files_shown = false;
			//show the documents popover
			files.clicked.connect (() => {
				if (files_shown)
					return;
				
				files_shown = true;
				
				var pop = new Widgets.PopOver ();
				var grid = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
				var src = new Granite.Widgets.SearchBar ("Search...");
				var scroll = new Gtk.ScrolledWindow (null, null);
				var files_view = new Widgets.FileView (this, file_view.file_model as Gtk.ListStore);
				var only_mine = new Gtk.Switch ();
				var mbox = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 0);
				
				grid.margin = 12;
				grid.width_request = 450;
				grid.height_request = 280;
				
				only_mine.halign = Gtk.Align.END;
				
				mbox.pack_start (new Widgets.LLabel (_("Show only my documents")));
				mbox.pack_start (only_mine);
				mbox.margin_top = 6;
				
				files_view.selected.connect ( () => pop.destroy () );
				scroll.margin_top = 6;
				
				scroll.add (files_view);
				scroll.hscrollbar_policy = Gtk.PolicyType.NEVER;
				
				grid.pack_start (src, false);
				grid.pack_start (mbox, false);
				grid.pack_start (scroll);
				
				only_mine.notify["active"].connect (() => {
					files_view.only_mine = only_mine.active;
					Services.Settings.get_default ().show_only_mine = only_mine.active;
				});
				only_mine.active = Services.Settings.get_default ().show_only_mine;
				src.text_changed_pause.connect ((text) => {
					files_view.filter_string = text;
				});
				
				files_view.filter.refilter ();
				
				(pop.get_content_area () as Gtk.Box).add (grid);
				pop.move_to_widget (files);
				pop.show_all ();
				pop.destroy.connect (() => files_shown = false );
				pop.hide.connect (() => files_shown = false );
			});
			
			//handle welcome screen
			welcome.activated.connect ((idx) => {
				switch (idx) {
					case 0:
						create_document (DocType.DOCUMENT);
						break;
					case 1:
						create_document (DocType.SPREADSHEET);
						break;
					case 2:
						create_document (DocType.PRESENTATION);
						break;
					default:
						break;
				}
			});
			
			//trash
			trash.clicked.connect (() => {
				try {
					var document = ((notebook.current.page as Gtk.ScrolledWindow).get_child () as Widgets.WebView).doc;
					if (docs.delete_entry (docs.get_primary_authorization_domain (), 
						docs.query_single_entry (docs.get_primary_authorization_domain (), document.id, null, new DocumentsText (null).get_type (), null), null)) {
						notebook.remove_tab (notebook.current);
					}
				} catch (Error e) { warning (e.message); }
			});
			
			//download
			download.clicked.connect (() => {
				
				var dlg = new Gtk.FileChooserDialog (_("Download File"), window, Gtk.FileChooserAction.SAVE, 
					Gtk.Stock.CANCEL, Gtk.ResponseType.CANCEL, Gtk.Stock.SAVE, Gtk.ResponseType.ACCEPT);
				
				var document = ((notebook.current.page as Gtk.ScrolledWindow).get_child () as Widgets.WebView);
				string [] types = {};
				
				switch (document.doc_type) {
					case DocType.SPREADSHEET:
						types = {DocumentsSpreadsheetFormat.CSV, DocumentsSpreadsheetFormat.HTML, DocumentsSpreadsheetFormat.ODS, 
							DocumentsSpreadsheetFormat.PDF, DocumentsSpreadsheetFormat.TSV, DocumentsSpreadsheetFormat.XLS};
						break;
					case DocType.PRESENTATION:
						types = {DocumentsPresentationFormat.PDF, DocumentsPresentationFormat.PNG, DocumentsPresentationFormat.PPT, 
							DocumentsPresentationFormat.SWF, DocumentsPresentationFormat.TXT};
						break;
					default: //DOCUMENT
						types = {DocumentsTextFormat.HTML, DocumentsTextFormat.DOC, DocumentsTextFormat.ODT, DocumentsTextFormat.PDF, 
							DocumentsTextFormat.PNG, DocumentsTextFormat.RTF, DocumentsTextFormat.TXT, DocumentsTextFormat.ZIP};
						break;
				}
				
				for (var i=0;i<types.length;i++) {
					var filter = new Gtk.FileFilter ();
					filter.set_filter_name ("."+types[i]);
					dlg.add_filter (filter);
				}
				
				dlg.set_current_name (document.doc.title);
				if (dlg.run () == Gtk.ResponseType.ACCEPT) {
					
					try {
						var mime = dlg.get_filter ().get_filter_name ();
						var dest = File.new_for_uri (dlg.get_uri ()+mime).replace (null, false, FileCreateFlags.REPLACE_DESTINATION, null);
						var down = document.doc.download (docs, mime.substring (1), null);
					
						dest.splice_async.begin (down, OutputStreamSpliceFlags.CLOSE_SOURCE | OutputStreamSpliceFlags.CLOSE_TARGET, 
							Priority.DEFAULT, null, () => {
							try {
								new Notify.Notification (document.doc.title+_("has been downloaded."), null, document.icon).show ();
							} catch (Error e) { warning (e.message); }
						});
						
					} catch (Error e) { warning (e.message); }
				
				}
				dlg.destroy ();
			});
			
			//delete the stored login data
			delete_login_data.activate.connect (clear_saved_data);
			
			//show the preferences dialog
			show_prefs.activate.connect (() => {
				var prefs = new Widgets.Preferences (this);
				prefs.run ();
				prefs.destroy ();
			});
			
			//check if all tabs are closed and remove quicklist entry
			notebook.tab_removed.connect ((tab) => {
				((tab.page as Gtk.ScrolledWindow).get_child () as Widgets.WebView).close ();
				
				if (notebook.n_tabs - 1 == 0) {
					trash.hide ();
					download.hide ();
					notebook.hide ();
					welcome.show ();
				}
				return true;
			});
			
			//setup cookies for password remembering
			WebKit.get_default_session ().add_feature (new Soup.CookieJarText (cookie_file.get_path (), false));
		}
		
		public void clear_saved_data ()
		{
			try {
				if (login_data_file.query_exists ()) {
					login_data_file.delete ();
				}
				if (cookie_file.query_exists ())
					cookie_file.delete ();
			} catch (Error e) { warning (e.message); }
		}
		
		public void open_document (DocumentsDocument doc, string icon_name="empty")
		{
			var scroll_view = new Gtk.ScrolledWindow (null, null);
			var tab = new Widgets.Tab (doc.title, new ThemedIcon (icon_name), scroll_view);
			
			var view = new Widgets.WebView (doc, hud, launcher, tab, window);
			scroll_view.add (view);
			
			notebook.insert_tab (tab, -1);
			
			notebook.show_all ();
			trash.show ();
			download.show ();
			access.show ();
			welcome.hide ();
		}
		
		public Entry? create_document (DocType type=DocType.DOCUMENT, string title = _("Untitled"))
		{
			DocumentsDocument doc;
			
			var icon = "x-office-document";
			switch (type) {
				case DocType.PRESENTATION:
					icon = "x-office-presentation";
					doc = new DocumentsPresentation (null);
					break;
				case DocType.SPREADSHEET:
					icon = "x-office-spreadsheet";
					doc = new DocumentsSpreadsheet (null);
					break;
				default:
					doc = new DocumentsText (null);
					break;
			}
			
			doc.title = "Untitled";
			Entry res = null;
			try {
				res = docs.insert_entry (docs.get_primary_authorization_domain (), docs.get_upload_uri (null), doc, null);
			
				open_document ((res as DocumentsDocument), icon);
			} catch (Error e) { warning (e.message); }
			
			return null;
		}
		
		/*client login in case GOA is not available*/
		public Authorizer? client_login ()
		{
			ClientLoginAuthorizer? auth = null;
			bool authenticated = false;
			
			//try to read stored data
			if (login_data_file.query_exists ()) {
				string data;
				try {
					GLib.FileUtils.get_contents (login_data_file.get_path (), out data);
				} catch (Error e) { warning (e.message); }
				var parts = data.split ("\n");
				
				auth = new ClientLoginAuthorizer ("drive", new DocumentsService (null).get_type ());
				auth.authenticate_async.begin (parts[0], parts[1], null, (obj, res) => {
					try {
						authenticated = auth.authenticate_async.end (res);
						authorization_done (auth);
						
					} catch (Error e) {
						warning ("Saved authentication failed: %s", e.message);
						
						var err = new Gtk.Dialog.with_buttons (_("Error while connecting"), window, Gtk.DialogFlags.MODAL);
						var grid = new Gtk.Grid ();
						
						var cancel = new Gtk.Button.with_label (_("Cancel"));
						var del_data = new Gtk.Button.with_label (_("Delete"));
						
						(err.get_content_area () as Gtk.Container).add (grid);
						
						grid.margin = 12;
						grid.attach (new Gtk.Image.from_stock (Gtk.Stock.DIALOG_ERROR, Gtk.IconSize.DIALOG), 0, 0, 1, 1);
						grid.attach (new Gtk.Label (_("There was an error connecting to your Google Account")+"\n"+
							_("Do you want to delete the stored login data?")), 1, 0, 1, 1);
						grid.attach (cancel, 0, 1, 1, 1);
						grid.attach (del_data, 1, 1, 1, 1);
						
						del_data.clicked.connect (() => {
							try {
								if (login_data_file.query_exists ()) {
									login_data_file.delete ();
								}
								if (cookie_file.query_exists ())
									cookie_file.delete ();
							} catch (Error e) { warning (e.message); }
							
							err.destroy ();
							Posix.exit (0);
						});
						
						cancel.clicked.connect (() => Posix.exit (0) );
						
						err.show_all ();
						err.run ();
					}
				});
				
				already_authenticated = true;
				
				return null;
			}
			
			//no stored data, ask user for authentication
			var dlg = new Gtk.Dialog.with_buttons (_("Connect"), window, Gtk.DialogFlags.MODAL);
			var grid = new Gtk.Grid ();
			
			var icon = new Gtk.Image.from_icon_name ("internet-web-browser", Gtk.IconSize.DIALOG);
			var working = new Gtk.Spinner ();
			
			var save = new Gtk.CheckButton.with_label (_("Save password"));
			var title = new Gtk.Label ("<b>"+_("Connect to your Google Account")+"</b>");
			var email = new Gtk.Entry ();
			var password = new Gtk.Entry ();
			var cancel = new Gtk.Button.with_label (_("Cancel"));
			var connect = new Gtk.Button.with_label (_("Connect"));
			
			save.tooltip_text = _("Note, passwords are currently saved as plain text!");
			
			password.visibility = false;
			
			working.height_request = working.width_request = 48;
			
			title.use_markup = true;
			
			grid.margin = 12;
			grid.column_spacing = 6;
			grid.row_spacing = 6;
			grid.attach (icon, 0, 0, 1, 1);
			grid.attach (title, 1, 0, 2, 1);
			grid.attach (new Gtk.Label (_("Email:")), 1, 1, 1, 1);
			grid.attach (email, 2, 1, 1, 1);
			grid.attach (new Gtk.Label (_("Password:")), 1, 2, 1, 1);
			grid.attach (password, 2, 2, 1, 1);
			grid.attach (save, 2, 3, 1, 1);
			grid.attach (cancel, 0, 5, 1, 1);
			grid.attach (connect, 2, 5, 1, 1);
			
			dlg.resizable = false;
			(dlg.get_content_area () as Gtk.Box).add (grid);
			grid.show_all ();
			
			password.activate.connect (() => connect.clicked () );
			
			dlg.destroy.connect (() => {
				if (!authenticated)
					Posix.exit (0);
			});
			
			cancel.clicked.connect (() => dlg.destroy ());
			connect.clicked.connect (() => {
				auth = new ClientLoginAuthorizer ("drive", new DocumentsService (null).get_type ());
				
				grid.sensitive = false;
				grid.remove (icon);
				grid.attach (working, 0, 0, 1, 1);
				working.start ();
				working.show ();
				
				auth.authenticate_async.begin (email.text, password.text, null, (obj, res) => {
					
					grid.remove (working);
					grid.attach (icon, 0, 0, 1, 1);
					icon.show ();
					grid.sensitive = true;
					
					try {
						authenticated = auth.authenticate_async.end (res);
						if (authenticated) {
							if (save.active) {
								try {
									GLib.FileUtils.set_contents (login_data_file.get_path (), email.text+"\n"+password.text);
								} catch (Error e) { warning (e.message); }
							}
							dlg.destroy ();
						}
					} catch (Error e) {
						message (e.message);
						
						var error = new Gtk.Label ("<span color='#f00'>"+_("Something went wrong, please try again.")+"</span>");
						error.use_markup = true;
						grid.attach (error, 1, 4, 2, 1);
						icon.icon_name = "dialog-error";
						grid.show_all ();
						
						email.grab_focus ();
					}
					
				});
			});
			
			dlg.run ();
			dlg.destroy ();
			
			if (!authenticated)
				return null;
			return auth;
		}
		
		public override void activate ()
		{
			debug ("Application activated");
			
			if (get_windows ().length () == 0)
				start_ui ();
			
			if (docs == null) {
				authorization_done.connect_after (() => {
					if (create_doc) {
						create_document (DocType.DOCUMENT);
						create_doc = false;
					}
					if (create_spread) {
						create_document (DocType.SPREADSHEET);
						create_spread = false;
					}
					if (create_pres) {
						create_document (DocType.PRESENTATION);
						create_pres = false;
					}
				});
			} else {
				if (create_doc) {
					create_document (DocType.DOCUMENT);
					create_doc = false;
				}
				if (create_spread) {
					create_document (DocType.SPREADSHEET);
					create_spread = false;
				}
				if (create_pres) {
					create_document (DocType.PRESENTATION);
					create_pres = false;
				}
			}
			
		}
		
		public override void open (File [] file_list, string hint)
		{
			debug ("Open request with %u files", file_list.length);
			
			if (file_list.length > 0)
				opening = true;
			else
				return;
			
			
			Notify.Notification notification = null;
			try {
				var exp = (file_list.length == 1) ? file_list[0].get_basename () : file_list.length.to_string ()+" "+_("Files");
				notification = new Notify.Notification ((_("Uploading")+" "+exp+"...").printf (file_list.length), null, "up");
				notification.show ();
			} catch (Error e) { warning (e.message); }
			
			hold ();
			
			var files = file_list;
			if (!authenticated)
				authorization_done.connect_after (() => continue_open_action (files, notification, true));
			else
				continue_open_action (files, notification, false);
		}
		
		//is called from two places depending on the state of authentication
		void continue_open_action (File [] files, Notify.Notification notification, bool no_ui_yet)
		{
			var done = 0;
			var failed = 0;
			launcher.progress = 0;
			launcher.progress_visible = true;
			launcher.count = files.length;
			launcher.count_visible = true;
			
			List<DocumentsDocument> results = new List<DocumentsDocument> ();
			
			foreach (var file in files) {
				done ++;
				launcher.progress = done / files.length;
				try {
					
					var info = file.query_info (FileAttribute.STANDARD_DISPLAY_NAME+","+FileAttribute.STANDARD_CONTENT_TYPE, 0);
					var stream = docs.upload_document (null, info.get_display_name (), info.get_content_type (), null, null);
					
					stream.splice (file.read (null), OutputStreamSpliceFlags.CLOSE_SOURCE | OutputStreamSpliceFlags.CLOSE_TARGET, null);
					results.append (docs.finish_upload (stream));
					
				} catch (Error e) {
					var start = e.message.index_of ("<internalReason>") + 16;
					var error = e.message.substring (start, e.message.index_of ("</internalReason>") - start);
					notification.update (_("Upload of %s failed").printf (file.get_basename ()), error, "dialog-error");
					failed ++;
					try {
						notification.show ();
					} catch (Error e) { warning (e.message); }
				}
			}
			
			if (!(failed == 1 && done == 1)) { //if there's one upload which failed, dont show done
				notification.update (_("Upload done"), "", "dialog-apply");
				try {
					notification.show ();
				} catch (Error e) { warning (e.message); }
			}
			
			launcher.progress_visible = false;
			launcher.count_visible = false;
			
			if (Services.Settings.get_default ().open_uploaded) {
				
				if (no_ui_yet) {
					start_ui ();
					window.sensitive = true;
					progress_item.hide ();
				}
				
				foreach (var res in results)
					open_document (res);
			}
			
			release ();
		}
	}
}

static bool create_doc = false;
static bool create_pres = false;
static bool create_spread = false;

static const OptionEntry [] options = {
	{"create-document", 'd', 0, OptionArg.NONE, ref create_doc, "Create a new document", ""},
	{"create-presentation", 'p', 0, OptionArg.NONE, ref create_pres, "Create a new presentation", null},
	{"create-spreadsheet", 's', 0, OptionArg.NONE, ref create_spread, "Create a new spreadsheet", null},
	{null}
};

public static void main (string [] args)
{
	var ctx = new OptionContext ("ctx");
	ctx.add_main_entries (options, "gwoffice");
	ctx.add_group (Gtk.get_option_group (true));
	
	try {
		ctx.parse (ref args);
	} catch (Error e) {
		error (e.message);
	}
	
	Gtk.init (ref args);
	
	if (!Notify.init ("GWoffice"))
		warning ("notifications not available");
	
	var app = new GWoffice.App ();
	Granite.Services.Logger.DisplayLevel = Granite.Services.LogLevel.DEBUG;
	
	app.run (args);
}

