using GData;

[CCode (cname = "oauth_gen_nonce")]
public extern string oauth_nonce ();
[CCode (cname = "oauth_sign_hmac_sha1")]
public extern string oauth_sign (string a, string b);
[CCode (cname = "oauth_url_escape")]
public extern string oauth_escape (string str);

namespace GWoffice.Services {
	
	public class GoaAuthorizer : GLib.Object, GData.Authorizer {
		
		Goa.Object goa_object;
		HashTable<AuthorizationDomain,AuthorizationDomain> auth_domains;
		string access_token;
		string access_token_secret;
		
		public GoaAuthorizer (Goa.Object _goa_object)
		{
			goa_object = _goa_object;
			auth_domains = new HashTable<AuthorizationDomain,AuthorizationDomain> (null, null);
			
			var list = Service.get_authorization_domains (new DocumentsService (null).get_type ());
			list.foreach ((l) => auth_domains.insert (l, l) );
		}
		
		public bool is_authorized_for_domain (AuthorizationDomain domain)
		{
			return auth_domains.lookup (domain) != null;
		}
		
		public void process_request (GData.AuthorizationDomain? domain, Soup.Message message)
		{
			if (auth_domains.lookup (domain) == null)
				return;
			if (access_token == null)
				return;
			
			var oauth = goa_object.get_oauth_based ();
			
			var parameters = new HashTable<string,string> (str_hash, str_equal);
			
			var uri = message.get_uri ();
			if (uri.get_query () != null) {
				var table = Soup.Form.decode (uri.get_query ());
				var iter = HashTableIter<string,string> (table);
				string key, val;
				while (iter.next (out key, out val))
					parameters.insert (key, val);
			}
			
			string query = "";
			
			var list = parameters.get_keys ();
			list.sort (strcmp);
			list.foreach ((k) => {
				var val = parameters.lookup (k);
				if (query != "")
					query += "&";
				query += oauth_escape (k) + "=" + oauth_escape (val);
			});
			
			uri = uri.copy ();
			uri.set_query (null);
			uri.set_fragment (null);
			var request = uri.to_string (false);
			
			var base_string = oauth_escape (message.method) + "&" + oauth_escape (request) + "&" + oauth_escape (query);
			
			var signing_key = oauth_escape (oauth.consumer_secret) + "&" + oauth_escape (access_token_secret);
			
			parameters.insert ("oauth_version", "1.0");
			parameters.insert ("oauth_nonce", oauth_nonce ());
			parameters.insert ("oauth_timestamp", new DateTime.now_local ().to_unix ().to_string ());
			parameters.insert ("oauth_consumer_key", oauth.consumer_key);
			parameters.insert ("oauth_token", access_token);
			parameters.insert ("oauth_signature_method", "HMAC-SHA1");
			parameters.insert ("oauth_signature", oauth_sign (base_string, signing_key));
			
			var authorization = "OAuth ";
			
			string [] oauth_keys = {
				"oauth_version",
				"oauth_nonce",
				"oauth_timestamp",
				"oauth_consumer_key",
				"oauth_token",
				"oauth_signature_method",
				"oauth_signature"
			};
			
			for (var i=0;i<oauth_keys.length;i++) {
				string key = oauth_keys[i];
				string val = parameters.lookup (key);
				
				if (i > 0)
					authorization += ", ";
				authorization += key;
				authorization += "=";
				authorization += "\"";
				authorization += oauth_escape (val);
				authorization += "\"";
			}
			
			message.request_headers.replace ("Authorization", authorization);
		}
		
		public bool refresh_authorization (GLib.Cancellable? cancellable) throws GLib.Error
		{
			bool success = true;
			access_token = null;
			access_token_secret = null;
			
			var acc = goa_object.get_account ();
			var oauth = goa_object.get_oauth_based ();
			
			try {
				success &= acc.call_ensure_credentials_sync (null, null);
				success &= oauth.call_get_access_token_sync (out access_token, 
					out access_token_secret, null, null);
			} catch (Error e) { warning (e.message); }
			
			return success;
		}
		
		public async bool refresh_authorization_async (GLib.Cancellable? cancellable)
		{
			return false;
		}
	}
}
